/* Copyright (c) 2007, 2008, Diego_Rocha
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Diego_Rocha nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COPYRIGHT ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COPYRIGHT BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
*
* Author: Diego_Rocha <diegorocha1987@gmail.com>
*
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <dirent.h>

#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <regex.h>

#include "pkgsearch.h"

int uflag, dflag, sflag, iflag;
char *home;

int main( int argc, char *argv[] )
{
	if( argc > 1 ) {
		int opt;
		uflag = dflag = sflag = iflag = 0;
		
		while( (opt = getopt( argc, argv, "udsihv" )) != -1 ) {
			switch(opt) {
				case 'u':
					uflag = 1;
					break;
				case 'd':
					dflag = 1;
					break;
				case 's':
					sflag = 1;
					break;
				case 'i':
					iflag = 1;
					break;
				case 'h':
					help();
				case 'v':
					version();
				default:
					usage();
			}
		}
		
		home = getenv( "HOME" );
		
		if( 1 == uflag ) {
			make_index_ports();
			make_index_pkg();
			return 0;
		}
			
		if( optind < argc ) {
			int index;
			for( index = optind; index < argc; index++ ) {
				if( 1 == iflag )
					search_pkg( argv[index] );
				else
					search_ports( argv[index] );
			}
		}
		else
			usage();
	}
	else
		usage();
	
	fflush( stdout );
	return 0;
}

int make_index_ports(void)
{
	DIR *dir;
	struct dirent *entry;
	struct stat status;
	FILE *portindex;
	char file[MAXPATHLEN];
	char curpath[MAXPATHLEN];
	unsigned int count = 0;
	
	(void)snprintf( file, MAXPATHLEN, "%s/%s", home, INDEXPORT );
	
	portindex = fopen( file, "w+" );
	
	if( NULL == portindex )
		ABORT("");
	
	if( getcwd( curpath, MAXPATHLEN+1 ) == NULL )
		ABORT("");
		
	dir = opendir( PORTS );
	if( NULL == dir )
		ABORT("");
	
	if( chdir( PORTS ) != 0 )
		ABORT( PORTS );
	
	while( ( entry = readdir( dir )) != NULL ) {
		if( (strncmp( entry->d_name, ".", 1) != 0) 
				   && (strncmp( entry->d_name, "..", 2) != 0)) {
			if( stat( entry->d_name, &status ) != 0 )
				ABORT( entry->d_name );
			
			if( S_ISDIR( status.st_mode ) 
						 && strncmp( entry->d_name, "distfiles", 9 ) != 0 
						 && strncmp( entry->d_name, "Mk", 2) != 0
						 && strncmp( entry->d_name, "Tools", 5) != 0 ) {
				DIR *colection;
				struct dirent *col_entry;
				struct stat col_status;
				
				colection = opendir( entry->d_name );
				if( colection == NULL ) 
					ABORT("");
				
				if( chdir( entry->d_name ) != 0 )
					ABORT( entry->d_name );
				
				while( ( col_entry = readdir( colection )) != NULL ) {
					count++;
					if( strncmp( col_entry->d_name, ".", 1) != 0 
									   && strncmp( col_entry->d_name, "..", 2) != 0 ) {
						if( stat( col_entry->d_name, &col_status ) != 0 )
							ABORT( col_entry->d_name );
						
						if( S_ISDIR( col_status.st_mode ) )
							(void)fprintf( portindex,"%s %s\n", entry->d_name, col_entry->d_name );
					}
				}
				if( closedir( colection ) != 0 ) 
					ABORT("");
					
				if( chdir( ".." ) != 0 )
					ABORT( ".." );	
			}
			
		}
	}
	
	(void)fprintf( stdout, "%d packages in %s\n", count, PORTS  );
	
	if( closedir( dir ) != 0 ) 
		ABORT("");
	
	if( chdir( curpath ) != 0 )
		ABORT( curpath );
	
	fclose( portindex );
	
	return 0;
}

int simple_getline( char *msg, size_t len, FILE *fp )
{
	char *ch;
	
	if( fp == NULL )
		return -1;
	memset( msg, 0, len );
	ch = msg;
	while( !feof( fp ) ) {
		*ch = getc( fp );
		if( *ch == '\n' || len == 0 )
			return 0;
		ch++;
		len--;
	}
	return -1;
}

int make_index_pkg(void)
{
	DIR *dir;
	struct dirent *entry;
	struct stat status;
	FILE *pkgindex;
	char file[MAXPATHLEN];
	char curpath[MAXPATHLEN];
	unsigned int count = 0;
	
	(void)snprintf( file, MAXPATHLEN, "%s/%s", home, INDEXPKG );
	
	pkgindex = fopen( file, "w+" );
	if( pkgindex == NULL )
		ABORT("");
	
	if( getcwd( curpath, MAXPATHLEN+1 ) == NULL )
		ABORT("");
		
	dir = opendir( PKG );
	if( dir == NULL )
		ABORT("");
	
	if( chdir( PKG ) != 0 )
		ABORT( PKG );
	
	while( ( entry = readdir( dir )) != NULL ) {
		count++;
		if( (strncmp( entry->d_name, ".", 1) != 0) 
				   && (strncmp( entry->d_name, "..", 2) != 0)) {
			if( stat( entry->d_name, &status ) != 0 )
				ABORT( entry->d_name );
			
			if( S_ISDIR( status.st_mode ) )
				(void)fprintf( pkgindex, "%s\n", entry->d_name );
		}
	}
	
	if( closedir( dir ) != 0 ) 
		ABORT("");
	
	if( chdir( curpath ) != 0 )
		ABORT( curpath );
	
	fclose( pkgindex );
	
	(void)fprintf( stdout, "%d packages in %s\n", count, PKG );
	
	return 0;
}

int regex_func( char *exp, char *string )
{
	size_t status;
	regex_t re;
	
	if(regcomp(&re, exp, REG_EXTENDED |  REG_ICASE ) != 0 )
		return 0;
	status = regexec(&re, string, (size_t)0, NULL, 0);
	regfree(&re);
	return status != 0 ? 1 : 0;
}

int search_ports( char *pkg )
{
	char *path;
	char *file;
	path = (char*)calloc(150, sizeof(char));
	file = (char*)calloc(150, sizeof(char));
	FILE *portindex;
	char filename[MAXPATHLEN];
	
	(void)snprintf( filename, MAXPATHLEN, "%s/%s", home, INDEXPORT );
	
	portindex = fopen( filename, "r" );
	if( portindex == NULL ) {
		(void)fprintf( stdout, "You don't have the %s, use the pkgsearch -u to make this\n",
		 filename );
		exit( 1 );
	}
	
	while( fscanf( portindex, "%s%s", path, file ) != EOF ) {
		if( regex_func( pkg, file ) == 0 ) {
			(void)fprintf( stdout, "%s/%s/%s\n", PORTS, path, file );
			char name[MAXPATHLEN];
			
			/* get the size of the package */
			if( sflag == 1 )
			{
				(void)snprintf( name, MAXPATHLEN, "%s/%s/%s/%s", PORTS, path, file, INFO );
				show_size( name );
				name[0] = '\0';
			}
			
			/* get the description of the package */
			if( dflag == 1 )
			{
				
				(void)snprintf( name, MAXPATHLEN, "%s/%s/%s/%s", PORTS, path, file, DESC );
				show_desc( name );
				name[0] = '\0';
			}
		}
	}
	
	free(path);
	free(file);
	fclose( portindex );
	return 0;
}

int search_pkg( char *pkg )
{
	char *file;
	FILE *pkgindex;
	char filename[MAXPATHLEN];
	char name[MAXPATHLEN];
	
	file = (char*)calloc(150, sizeof(char));
	(void)snprintf( filename, MAXPATHLEN, "%s/%s", home, INDEXPKG );
	
	pkgindex = fopen( filename, "r" );
	if( pkgindex == NULL ) {
		(void)fprintf( stdout, "You don't have the %s use the pkgsearch -u to make this\n", 
		 filename );
		exit( 1 );
	}
	
	while( fscanf( pkgindex, "%s", file ) != EOF ) {
		if( regex_func( pkg, file ) == 0 ) {
			(void)fprintf( stdout, "%s\n", file );
			
			/* get the description */
			if( dflag == 1 ) {
				(void)snprintf( name, MAXPATHLEN, "%s/%s/%s", PKG, file, PKGDESC );
				show_desc( name );
				name[0] = '\0';
			}
		}
	}
	
	free(file);
	fclose( pkgindex );
	return 0;
}

int show_desc( char *pkg )
{
	FILE *desc;
	char *msg;
	
	desc = fopen( pkg, "r" );
	if( desc == NULL ) {
		(void)fprintf( stdout, "\t%s no such file\n", pkg );
		return 0;
	}
	
	msg = (char*)malloc( sizeof(char)*MAXLINELEN);
	
	(void)fprintf( stdout, "DESC: \n" );
	while( simple_getline( msg, MAXLINELEN, desc ) != -1 )
		if( msg[0] != '#' )
			(void)fprintf( stdout, "\t%s", msg );
	(void)fprintf( stdout, "\n" );
	fflush( stdout );
	free( msg );
	return 0;
}

int show_size( char *pkg )
{
	FILE *info;
	char *msg;
	
	info = fopen( pkg, "r" );
	if( info == NULL ) {
		(void)fprintf( stdout, "\t%s no such file\n\n", pkg );
		return 0;
	}
	
	msg = (char*)malloc( sizeof(char)*MAXLINELEN);
	
	while( simple_getline( msg, MAXLINELEN, info ) != -1 )
		if( msg[0] != '#' )
			if( strncmp( msg, "SIZE", 4 ) == 0 )
				(void)fprintf( stdout, "\t%s", msg );
	(void)fprintf( stdout, "\n" );
	fflush( stdout );
	free( msg );
	return 0;
}

void usage()
{
	(void)fprintf( stdout, "usage: pkgsearch [-u][-h][-v][-dis] packname...\n\n");
	exit(1);
}

void help()
{
	(void)fprintf( stdout, "usage: pkgsearch [-u][-h][-v][-dis] packname...\n");
	(void)fprintf( stdout, "\tu\t\tupdate the database\n");
	(void)fprintf( stdout, "\td\t\tget the description of the package\n");
	(void)fprintf( stdout, "\ts\t\tget the size of the package *not work with -i*\n");
	(void)fprintf( stdout, "\ti\t\tsearch in packages installeds\n");
	(void)fprintf( stdout, "\th\t\tshow this\n");
	(void)fprintf( stdout, "\tv\t\tshow the version\n\n");
	exit( 1 );
}

void version()
{
	(void)fprintf( stdout, "Version is %s\n", VERSION );
	(void)fprintf( stdout, "Copyright (C) 2007, 2008 license BSD.\n");
	(void)fprintf( stdout, "pkgsearch is Developed for Diego_Rocha <diegorocha1987@gmail.com> \n");
	(void)fprintf( stdout, "http://pkgsearch.sharesource.org\n\n" );
	exit( 1 );
}
